package com.xxfc.platform.order.biz.inner;

import cn.hutool.core.date.DateUtil;
import com.github.wxiaoqi.security.admin.feign.UserFeign;
import com.github.wxiaoqi.security.admin.feign.dto.AppUserDTO;
import com.xxfc.platform.app.entity.Cofig;
import com.xxfc.platform.app.feign.ConfigFeign;
import com.xxfc.platform.order.biz.OrderRefundBiz;
import com.xxfc.platform.order.contant.enumerate.OrderTypeEnum;
import com.xxfc.platform.order.contant.enumerate.RefundTypeEnum;
import com.xxfc.platform.order.entity.BaseOrder;
import com.xxfc.platform.order.entity.OrderMemberDetail;
import com.xxfc.platform.order.entity.OrderRentVehicleDetail;
import com.xxfc.platform.order.entity.OrderTourDetail;
import com.xxfc.platform.universal.constant.DictionaryKey;
import com.xxfc.platform.universal.dto.SmsTemplateDTO;
import com.xxfc.platform.universal.entity.Dictionary;
import com.xxfc.platform.order.entity.OrderRefund;
import com.xxfc.platform.universal.feign.ThirdFeign;
import com.xxfc.platform.vehicle.entity.BranchCompany;
import com.xxfc.platform.vehicle.feign.VehicleFeign;
import com.xxfc.platform.vehicle.pojo.CompanyDetail;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.github.wxiaoqi.security.common.constant.CommonConstants.SYS_FALSE;
import static com.xxfc.platform.universal.constant.DictionaryKey.APP_ORDER;

/**
 * 订单退款记录表
 *
 * @author zjw
 * @email nishijjo@qq.com
 * @date 2019-06-18 11:08:09
 */
@Slf4j
@Service
public class OrderMsgBiz {

    //客户通知
    public static final int RENT_PAY = 1;
    public static final int RENT_DELIVERY = 2;
    public static final int RENT_CANCEL = 3;


    //
    public static final int PAY = 1;
    public static final int CANCEL = 2;

    //客户通知2
//    public static final int RENT_APP_A = 4;
//    public static final int RENT_APP_B = 5;
//    public static final int TOUR_APP = 6;
//    public static final int MEMBER_APP = 7;

    //参数
    public static final String USER_N = "尊敬的客户";
    public static final String USER_M = "尊贵的会员";


    @Autowired
    UserFeign userFeign;

    @Autowired
    ThirdFeign thirdFeign;

    @Autowired
    VehicleFeign vehicleFeign;

    @Autowired
    ConfigFeign configFeign;

    @Autowired
    OrderRefundBiz orderRefundBiz;

    /**
     * 处理App用户提醒短信的发送
     * @param orvd
     * @param omd
     * @param baseOrder
     * @param appUserDTO
     */
    public void handelAppUserMsg(OrderRentVehicleDetail orvd, OrderMemberDetail omd, BaseOrder baseOrder, AppUserDTO appUserDTO) {
        try {
            OrderTypeEnum orderTypeEnum = OrderTypeEnum.codeAndDesc.get(baseOrder.getType());
            Integer smstype;
            List<String> smsParams = new ArrayList<String>();
            smsParams.add(baseOrder.getRealAmount().toString());
            switch (orderTypeEnum) {
                case RENT_VEHICLE:
                    if(orvd.getFreeDays() > 0) {
                        smstype = SmsTemplateDTO.RENT_MEMENT;
                        smsParams.add(orvd.getFreeDays().toString());
                        smsParams.add(appUserDTO.getRentFreeDays().toString());
                    }else {
                        smstype = SmsTemplateDTO.RENT_NORMAL;
                    }
                    thirdFeign.sendTemplate(new SmsTemplateDTO(){{
                        setPhoneNumbers(appUserDTO.getUsername());
                        setType(smstype);
                        setParams(smsParams.toArray(new String[smsParams.size()]));
                    }});
                    break;
                case TOUR:
                    thirdFeign.sendTemplate(new SmsTemplateDTO(){{
                        setPhoneNumbers(appUserDTO.getUsername());
                        setType(SmsTemplateDTO.TOUR);
                        setParams(smsParams.toArray(new String[smsParams.size()]));
                    }});
                    break;
                case MEMBER:
                    smsParams.add(omd.getRentFreeNum().toString());
                    smsParams.add(appUserDTO.getRentFreeDays().toString());
                    thirdFeign.sendTemplate(new SmsTemplateDTO(){{
                        setPhoneNumbers(appUserDTO.getUsername());
                        setType(SmsTemplateDTO.MEMENT);
                        setParams(smsParams.toArray(new String[smsParams.size()]));
                    }});
                    break;
                default:
                    break;
            }
        }catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 处理后台用户提醒短信的发送
     * @param orvd
     * @param baseOrder
     * @param appUserDTO
     */
    public void handelBgUserMsg4Pay(OrderRentVehicleDetail orvd, BaseOrder baseOrder, AppUserDTO appUserDTO, Integer handelType) {
        try {
            OrderTypeEnum orderTypeEnum = OrderTypeEnum.codeAndDesc.get(baseOrder.getType());
            Integer smstype;
            List<String> smsParams = new ArrayList<String>();
            switch (orderTypeEnum) {
                case RENT_VEHICLE:
                    switch (handelType) {
                        case RENT_PAY:
                            //如果公司相同
                            if(null != orvd.getStartCompanyId() && !SYS_FALSE.equals(orvd.getStartCompanyId())) {
                                CompanyDetail companyDetail = vehicleFeign.getCompanyDetail(orvd.getStartCompanyId()).getData();
                                if(orvd.getStartCompanyId().equals(orvd.getEndCompanyId())) {
                                    smstype = SmsTemplateDTO.TAAKE_CAR;
                                }else {
                                    smstype = SmsTemplateDTO.DIFFERENT_TAAKE_CAR;
                                }
                                handelSmsParam(orvd, baseOrder, smsParams, appUserDTO, RENT_PAY);
                                SmsTemplateDTO smsTemplateDTO = new SmsTemplateDTO(){{
                                    setPhoneNumbers(companyDetail.getVehiceServicePhone());
                                    setType(smstype);
                                    setParams(smsParams.toArray(new String[smsParams.size()]));
                                }};
                                thirdFeign.sendTemplate(smsTemplateDTO);
                            }else {
                                log.error("通知后台客户短信异常，开始公司id 不存在或者为0 订单号：{}", baseOrder.getId());
                            }
                            break;
                        case RENT_DELIVERY:
                            //如果公司相同
                            if(null != orvd.getEndCompanyId() && !SYS_FALSE.equals(orvd.getEndCompanyId())) {
                                CompanyDetail companyDetail = vehicleFeign.getCompanyDetail(orvd.getEndCompanyId()).getData();
                                //不相同发送，相同不用发送
                                if(!orvd.getEndCompanyId().equals(orvd.getStartCompanyId())) {
                                    handelSmsParam(orvd, baseOrder, smsParams, appUserDTO, RENT_DELIVERY);
                                    thirdFeign.sendTemplate(new SmsTemplateDTO(){{
                                        setPhoneNumbers(companyDetail.getVehiceServicePhone());
                                        setType(SmsTemplateDTO.ALSO_CAR);
                                        setParams(smsParams.toArray(new String[smsParams.size()]));
                                    }});
                                }
                            }else {
                                log.error("通知后台客户短信异常，结束公司id 不存在或者为0 订单号：{}", baseOrder.getId());
                            }
                            break;
                        case RENT_CANCEL:
                            if(null != orvd.getEndCompanyId() && !SYS_FALSE.equals(orvd.getEndCompanyId())) {
                                handelSmsParam(orvd, baseOrder, smsParams, appUserDTO, RENT_CANCEL);
                                CompanyDetail companyDetail2 = vehicleFeign.getCompanyDetail(orvd.getEndCompanyId()).getData();
                                SmsTemplateDTO smsTemplateDTO = new SmsTemplateDTO();
                                smsTemplateDTO.setPhoneNumbers(companyDetail2.getVehiceServicePhone());
                                smsTemplateDTO.setType(Integer.valueOf(SmsTemplateDTO.CANCEL));
                                List<String> smsParams2 = smsParams;
                                smsTemplateDTO.setParams(smsParams2.toArray(new String[smsParams2.size()]));
                                thirdFeign.sendTemplate(smsTemplateDTO);
                            }else {
                                log.error("通知后台客户短信异常，结束公司id 不存在或者为0 订单号：{}", baseOrder.getId());
                            }
                            break;
                        default:
                            break;
                    }
                    break;
                default:
                    break;
            }
        }catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 支付订单处理提醒短信的发送
     * @param orvd
     * @param baseOrder
     * @param appUserDTO
     */
    public void handelMsgPayNew(OrderRentVehicleDetail orvd, OrderTourDetail otd, OrderMemberDetail omd, BaseOrder baseOrder, AppUserDTO appUserDTO) {
        try {
            OrderTypeEnum orderTypeEnum = OrderTypeEnum.codeAndDesc.get(baseOrder.getType());
            Integer smstype;
            List<String> smsParams = new ArrayList<String>();
            switch (orderTypeEnum) {
                case RENT_VEHICLE:
                    CompanyDetail startCompanyDetail = vehicleFeign.getCompanyDetail(orvd.getStartCompanyId()).getData();
                    CompanyDetail endCompanyDetail = vehicleFeign.getCompanyDetail(orvd.getEndCompanyId()).getData();
                    if(null != orvd.getStartCompanyId() && !SYS_FALSE.equals(orvd.getStartCompanyId())) {
                        //使用免费天数
                        if(null != orvd.getFreeDays() && orvd.getFreeDays() > 0) {
                            smstype = SmsTemplateDTO.PAY_B;
                        }else {
                            smstype = SmsTemplateDTO.PAY_A;
                        }
                        sms2AppUser(orvd, otd, omd, baseOrder, appUserDTO, smstype, smsParams);

                        //后台发送消息(客服)
                        Cofig cofig = configFeign.getAllByType(ConfigFeign.TYPE_CUS_SER+ "").getData().get(0);
                        sms2BgUser(cofig.getValue(), startCompanyDetail, endCompanyDetail, orvd, baseOrder, appUserDTO, SmsTemplateDTO.PAY_C, smsParams);

                        //后台发送消息(出车人)
                        sms2BgUser(startCompanyDetail.getVehiceServicePhone(), startCompanyDetail, endCompanyDetail, orvd, baseOrder, appUserDTO, SmsTemplateDTO.PAY_D, smsParams);

                        //后台发送消息(收车人)
                        sms2BgUser(endCompanyDetail.getVehiceServicePhone(), startCompanyDetail, endCompanyDetail, orvd, baseOrder, appUserDTO, SmsTemplateDTO.PAY_D, smsParams);

                    }else {
                        log.error("通知短信异常，开始公司id 不存在或者为0 订单号：{}", baseOrder.getId());
                    }
                    break;
                case TOUR:
                    sms2AppUser(orvd, otd, omd, baseOrder, appUserDTO, SmsTemplateDTO.PAY_F, smsParams);
                    break;
                case MEMBER:
                    sms2AppUser(orvd, otd, omd, baseOrder, appUserDTO, SmsTemplateDTO.PAY_G, smsParams);
                    break;
                default:
                    break;
            }
        }catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 取消订单处理提醒短信的发送
     * @param orvd
     * @param baseOrder
     * @param appUserDTO
     */
    public void handelMsgCancel(OrderRentVehicleDetail orvd, OrderTourDetail otd, OrderMemberDetail omd, BaseOrder baseOrder, AppUserDTO appUserDTO) {
        try {
            OrderTypeEnum orderTypeEnum = OrderTypeEnum.codeAndDesc.get(baseOrder.getType());
            Integer smstype;
            List<String> smsParams = new ArrayList<String>();
            switch (orderTypeEnum) {
                case RENT_VEHICLE:
                    if(null != orvd.getEndCompanyId() && !SYS_FALSE.equals(orvd.getEndCompanyId())) {
                        //查询押金退款记录
                        OrderRefund orderRefund = orderRefundBiz.selectOne(new OrderRefund(){{
                            setOrderId(baseOrder.getId());
                            setRefundType(RefundTypeEnum.DEPOSIT.getCode());
                        }});

//                        if(BigDecimal.ZERO.equals(orderRefund))
                            //使用免费天数
                            if(null != orvd.getFreeDays() && orvd.getFreeDays() > 0) {
                                smstype = SmsTemplateDTO.CANCEL_B;
                            }else {
                                smstype = SmsTemplateDTO.CANCEL_A;
                            }

                        sms2AppUser(orvd, otd, omd, baseOrder, appUserDTO, smstype, smsParams);

                        //后台发送消息(出车人)
//                        sms2BgUser(startCompanyDetail.getVehiceServicePhone(), startCompanyDetail, endCompanyDetail, orvd, baseOrder, appUserDTO, SmsTemplateDTO.CANCEL_E, smsParams);
                    }else {
                        log.error("通知后台客户短信异常，结束公司id 不存在或者为0 订单号：{}", baseOrder.getId());
                    }
                    break;
                case TOUR:
                    break;
                case MEMBER:
                    break;
                default:
                    break;
            }
        }catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void sms2AppUser(OrderRentVehicleDetail orvd, OrderTourDetail otd, OrderMemberDetail omd, BaseOrder baseOrder, AppUserDTO appUserDTO, Integer smstype, List<String> smsParams) {
        handelSmsParamApp(orvd, otd, omd, baseOrder, smsParams, appUserDTO, smstype);
        SmsTemplateDTO smsTemplateDTO = new SmsTemplateDTO(){{
            setPhoneNumbers(appUserDTO.getUsername());
            setType(smstype);
            setParams(smsParams.toArray(new String[smsParams.size()]));
        }};
        thirdFeign.sendTemplate(smsTemplateDTO);
    }
    private void sms2RefundAppUser(BigDecimal originalAmount, BigDecimal violateAmount, BigDecimal residueAmount, BaseOrder baseOrder, AppUserDTO appUserDTO, Integer smstype, List<String> smsParams) {
        handelDepositSmsParamApp(originalAmount, violateAmount, residueAmount, baseOrder, smsParams, appUserDTO, smstype);
        SmsTemplateDTO smsTemplateDTO = new SmsTemplateDTO(){{
            setPhoneNumbers(appUserDTO.getUsername());
            setType(smstype);
            setParams(smsParams.toArray(new String[smsParams.size()]));
        }};
        thirdFeign.sendTemplate(smsTemplateDTO);
    }


    private void sms2BgUser(String phone,BranchCompany sCompany, BranchCompany eCompany, OrderRentVehicleDetail orvd, BaseOrder baseOrder, AppUserDTO appUserDTO, Integer smstype, List<String> smsParams) {
        handelSmsParamBg(sCompany, eCompany, orvd, baseOrder, smsParams, appUserDTO, smstype);
        SmsTemplateDTO smsTemplateDTO = new SmsTemplateDTO(){{
            setPhoneNumbers(phone);
            setType(smstype);
            setParams(smsParams.toArray(new String[smsParams.size()]));
        }};
        thirdFeign.sendTemplate(smsTemplateDTO);
    }

    private void handelSmsParam(OrderRentVehicleDetail orvd, BaseOrder baseOrder, List<String> smsParams, AppUserDTO appUserDTO, int handelType) {
        if(RENT_DELIVERY == handelType) {
            smsParams.add(appUserDTO.getRealname());
            smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getEndTime())));
            smsParams.add(orvd.getEndAddr());
            smsParams.add(baseOrder.getName());
            smsParams.add(appUserDTO.getUsername());
        }else {
            smsParams.add(appUserDTO.getRealname());
            smsParams.add(baseOrder.getName());
            smsParams.add(orvd.getStartAddr());
            smsParams.add(orvd.getEndAddr());
            smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
            smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getEndTime())));
            smsParams.add(orvd.getDayNum().toString());
            smsParams.add(appUserDTO.getUsername());
        }
    }

    private void handelSmsParamApp(OrderRentVehicleDetail orvd, OrderTourDetail otd, OrderMemberDetail omd, BaseOrder baseOrder, List<String> smsParams, AppUserDTO appUserDTO, int paramHandelType) {
        switch (paramHandelType) {
            case SmsTemplateDTO.PAY_A :
                if(0 < baseOrder.getMemberLevel() ) {
                    smsParams.add(USER_M);
                }else {
                    smsParams.add(USER_N);
                }
                smsParams.add(baseOrder.getRealAmount().toString());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(orvd.getStartAddr());
                break;
            case SmsTemplateDTO.PAY_B :
                smsParams.add(orvd.getFreeDays().toString());
                smsParams.add(appUserDTO.getRentFreeDays().toString());
                smsParams.add(baseOrder.getRealAmount().toString());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(orvd.getStartAddr());
                break;
            case SmsTemplateDTO.PAY_F :
                smsParams.add(baseOrder.getName());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(otd.getStartTime())));
                smsParams.add(otd.getStartAddr());
                break;
            case SmsTemplateDTO.PAY_G :
                smsParams.add(baseOrder.getRealAmount().toString());
                smsParams.add(omd.getRentFreeNum().toString());
                smsParams.add(appUserDTO.getRentFreeDays().toString());
                break;
            case SmsTemplateDTO.CANCEL_A :
                if(0 < baseOrder.getMemberLevel() ) {
                    smsParams.add(USER_M);
                }else {
                    smsParams.add(USER_N);
                }
                break;
            default:
                break;
        }
    }

    private void handelDepositSmsParamApp(BigDecimal originalAmount, BigDecimal violateAmount, BigDecimal residueAmount, BaseOrder baseOrder, List<String> smsParams, AppUserDTO appUserDTO, int paramHandelType) {
        switch (paramHandelType) {
            case SmsTemplateDTO.CANCEL_C :
                if(0 < baseOrder.getMemberLevel() ) {
                    smsParams.add(USER_M);
                }else {
                    smsParams.add(USER_N);
                }
                smsParams.add(violateAmount.toString());
                smsParams.add(residueAmount.toString());
                break;
            case SmsTemplateDTO.CANCEL_D :
                smsParams.add(violateAmount.toString());
                smsParams.add(residueAmount.toString());
                break;
            case SmsTemplateDTO.REFUND_A:
                Map<String, Dictionary> dictionaryMap = thirdFeign.dictionaryGetAll4Map().getData();
                Integer rentDepositAutoRefundTime = new Integer(dictionaryMap.get(APP_ORDER+ "_"+ DictionaryKey.RENT_DEPOSIT_AUTO_REFUND_TIME).getDetail());
                if(0 < baseOrder.getMemberLevel() ) {
                    smsParams.add(USER_M);
                }else {
                    smsParams.add(USER_N);
                }
                smsParams.add(baseOrder.getNo());
                smsParams.add(originalAmount.toString());
                smsParams.add(violateAmount.toString());
                smsParams.add(residueAmount.toString());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(baseOrder.getRefundTime() + Long.valueOf(rentDepositAutoRefundTime * 60 * 60 * 1000))));
                break;
            case SmsTemplateDTO.REFUND_B:
                if(0 < baseOrder.getMemberLevel() ) {
                    smsParams.add(USER_M);
                }else {
                    smsParams.add(USER_N);
                }
                smsParams.add(baseOrder.getNo());
                smsParams.add(originalAmount.toString());
                smsParams.add(violateAmount.toString());
                smsParams.add(residueAmount.toString());
                break;
            default:
                break;
        }
    }

    private void handelSmsParamBg(BranchCompany sCompany, BranchCompany eCompany, OrderRentVehicleDetail orvd, BaseOrder baseOrder, List<String> smsParams, AppUserDTO appUserDTO, int paramHandelType) {
        switch (paramHandelType) {
            case SmsTemplateDTO.PAY_C :
                smsParams.add(appUserDTO.getRealname());
                smsParams.add(appUserDTO.getUsername());
                smsParams.add(baseOrder.getName());
                smsParams.add(sCompany.getName());
                smsParams.add(sCompany.getVehiceServicePhone());
                smsParams.add(eCompany.getName());
                smsParams.add(eCompany.getVehiceServicePhone());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getEndTime())));
                smsParams.add(orvd.getDayNum().toString());
                break;
            case SmsTemplateDTO.PAY_D :
                smsParams.add(appUserDTO.getRealname());
                smsParams.add(appUserDTO.getUsername());
                smsParams.add(baseOrder.getName());
                smsParams.add(sCompany.getName());
                smsParams.add(eCompany.getName());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getEndTime())));
                smsParams.add(orvd.getDayNum().toString());
                break;
            case SmsTemplateDTO.PAY_E :
                smsParams.add(appUserDTO.getRealname());
                smsParams.add(appUserDTO.getUsername());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(sCompany.getName());
                smsParams.add(baseOrder.getName());
                break;
            case SmsTemplateDTO.CANCEL_E :
                smsParams.add(appUserDTO.getRealname());
                smsParams.add(appUserDTO.getUsername());
                smsParams.add(baseOrder.getName());
                smsParams.add(sCompany.getName());
                smsParams.add(eCompany.getName());
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getStartTime())));
                smsParams.add(DateUtil.formatDateTime(DateUtil.date(orvd.getEndTime())));
                smsParams.add(orvd.getDayNum().toString());
                break;
            default:
                break;
        }
    }
}